@@ -2,8 +2,8 @@
 #
 # read.cross.mm.R
 #
-# copyright (c) 2000-8, Karl W Broman
-# last modified Jun, 2008
+# copyright (c) 2000-9, Karl W Broman
+# last modified Sep, 2009
 # first written Aug, 2000
 #
 #     This program is free software; you can redistribute it and/or
@@ -74,7 +74,7 @@ function(dir,rawfile,mapfile,estimate.map=TRUE)
     if(length(o) > 0) map <- map[-o,]
 
     # remove any leading *'s from the marker names
-    g <- grep("^*",map[,2],extended=FALSE)
+    g <- grep("^\\*",map[,2],extended=FALSE)
     if(length(g) > 0)
       map[g,2] <- substr(map[g,2],2,nchar(map[g,2]))
   }
@@ -341,7 +341,7 @@ function( mapsfile )
   f <- scan( mapsfile, what = "", skip = start[3],
             nlines = start[4] - start[3] - 1,
             blank.lines.skip = FALSE, quiet = TRUE)
-  chr <- grep( "^*", f, extended = FALSE )
+  chr <- grep( "^\\*", f, extended = FALSE )
   chrom <- substring( f[chr], 2 )
   nmark <- as.integer( f[ 1 + chr ] )
   chr <- c( chr[-1], 1 + length( f ))
