@@ -2,22 +2,21 @@
 #
 # summary.cross.R
 #
-# copyright (c) 2001-8, Karl W Broman
-# last modified Dec, 2008
+# copyright (c) 2001-9, Karl W Broman
+# last modified Apr, 2009
 # first written Feb, 2001
 #
 #     This program is free software; you can redistribute it and/or
-#     modify it under the terms of the GNU General Public License, as
-#     published by the Free Software Foundation; either version 2 of
-#     the License, or (at your option) any later version. 
+#     modify it under the terms of the GNU General Public License,
+#     version 3, as published by the Free Software Foundation.
 # 
 #     This program is distributed in the hope that it will be useful,
 #     but without any warranty; without even the implied warranty of
-#     merchantability or fitness for a particular purpose.  See the
-#     GNU General Public License for more details.
+#     merchantability or fitness for a particular purpose.  See the GNU
+#     General Public License, version 3, for more details.
 # 
-#     A copy of the GNU General Public License is available at
-#     http://www.r-project.org/Licenses/
+#     A copy of the GNU General Public License, version 3, is available
+#     at http://www.r-project.org/Licenses/GPL-3
 # 
 # Part of the R/qtl package
 # Contains: summary.cross, print.summary.cross, nind, nchr, nmar,
@@ -39,14 +38,11 @@ function(object,...)
   type <- class(object)[1]
 
   if(type != "f2" && type != "bc" && type != "4way" &&
-     type != "riself" && type != "risib" && type != "cc" && type != "dh") 
+     type != "riself" && type != "risib" && type != "dh"
+     && type != "ri4self" && type != "ri4sib"
+     && type != "ri8self" && type != "ri8sib")
     stop("Cross type ", type, " is not supported.")
 
-  if(type=="cc") {
-    cat("A Collaborative cross\n")
-    return(NULL)
-  }
-
   # combine genotype data into one big matrix
   Geno <- pull.geno(object)
 
@@ -151,7 +147,6 @@ function(object,...)
   if(jitterwarning)
     warning("Some markers at the same position; use jittermap().")
 
-
   if(!is.data.frame(object$pheno)) 
     warning("Phenotypes should be a data.frame.")
 
@@ -234,7 +229,17 @@ function(object,...)
       warning(warn)
     }
   }
-
+  else if(type=="ri4sib" || type=="ri4self" || type=="ri8sib" || type=="ri8self") {
+    n.str <- as.numeric(substr(type, 3, 3))
+    if(any(!is.na(Geno) & (Geno != round(Geno) | Geno < 1 | Geno > 2^n.str-1))) {
+      u <- unique(as.numeric(Geno))
+      u <- sort(u[!is.na(u)])
+      warn <- paste("Invalid genotypes.",
+                    "\n    Observed genotypes:",
+                    paste(u, collapse=" "))
+      warning(warn)
+    }
+  }
 
   # Look for duplicate marker names
   mnames <- NULL
@@ -305,13 +310,22 @@ function(object,...)
 print.summary.cross <-
 function(x,...)
 {
-#  cat("\n")
+  print.genotypes <- TRUE
+
   if(x$type=="f2") cat("    F2 intercross\n\n")
   else if(x$type=="bc") cat("    Backcross\n\n")
   else if(x$type=="4way") cat("    4-way cross\n\n")
   else if(x$type=="riself") cat("    RI strains via selfing\n\n")
   else if(x$type=="risib") cat("    RI strains via sib matings\n\n")
   else if(x$type=="dh") cat("    Doubled haploids\n\n")
+  else if(x$type=="ri4self" || x$type=="ri4sib" || x$type=="ri8self" ||
+          x$type=="ri8sib") {
+    n.str <- substr(x$type, 3, 3)
+    if(substr(x$type, 4, nchar(x$type))=="sib") crosstype <- "sib-mating"
+    else crosstype <- "selfing"
+    print.genotypes <- FALSE
+    cat("    ", n.str, "-way RIL by ", crosstype, "\n\n", sep="")
+  }
   else cat("    cross", x$type, "\n\n",sep=" ")
 
   cat("    No. individuals:   ", x$n.ind,"\n\n")
@@ -364,10 +378,12 @@ function(x,...)
   cat("    No. markers:       ")
   printnicely(x$n.mar, header, width)
   cat("    Percent genotyped: ", round((1-x$missing.gen)*100,1), "\n")
-  cat("    Genotypes (%):    ")
-  geno <- paste(names(x$typing.freq),round(x$typing.freq*100,1),sep=":")
-  header <- "                      "
-  printnicely(geno, header, width, "  ")
+  if(print.genotypes) {
+    cat("    Genotypes (%):    ")
+    geno <- paste(names(x$typing.freq),round(x$typing.freq*100,1),sep=":")
+    header <- "                      "
+    printnicely(geno, header, width, "  ")
+  }
 }
 
 
